//*******************************************************************************
//* Dieses Programm wurde von Ulrich Radig fr Atmel AVR C geschrieben 	 	*
//* 																			*
//* Ich wnsche jeden viel Spa mit diesen kleinen Web-Server der ohne 			*
//* zustzlichen Speicher auskommt.												*
//*																				*
//* Bei Fragen und Verbesserungen wendet euch per EMail an						*
//*																				*
//* mail@ulrichradig.de															*
//*																				*
//* oder im Forum meiner Web Page : www.ulrichradig.de							*
//*																				*
//*******************************************************************************

#include <ethernet.h>

//Prototype
int get (void);
SIGNAL (SIG_INTERRUPT4);

SIGNAL (SIG_INTERRUPT4)
{
	//Globale Interrupts Ausschalten
	cli ();
	
	char ByteH = 0;
	char ByteL = 1;		

	//if the receive buffer has been overrun
	if ( (ReadRTL(ISR)&(1<<OVW)) != 0)
		{
		printf ("Overrun!\n");
		}
		
	//gehe zu Routine GET wenn ein Packet empfangen wurde
	if ( (ReadRTL(ISR) & (1<<PRX)) != 0)
		{
		//NIC FIFO noch nicht leer
		//gehe zu Routine GET und lese ein weiteres Packet
		while (ByteL != ByteH) //(!= bedeutet ungleich)
			{
			get ();
			
			//auslesen des Empfangsbuffer BNRY = CURR 
			ByteL = ReadRTL(BNRY); //auslesen NIC Register bnry
			WriteRTL ( CR , 0x62);

			ByteH = ReadRTL(CURR); //auslesen NIC Register curr
			WriteRTL ( CR , 0x22);
			} 
		}
		
	//Zurcksetzen der Interrupt Bits des NIC
	WriteRTL (ISR, 0xFF);
	
	//startet die Network Card
	WriteRTL (CR , 0x22);
	
	//Globale wieder Interrupts Einschalten
	sei ();
}

//Hier werden daten empfangen und geprft was empfangen wurde 
int get (void)
{
	//lnge des Ethernetpacketes
	unsigned int packetlen;

	//Array fr zu empfangene Daten und zu sendene Daten
	unsigned char packet[1500];

	//Lese daten aus NIC FIFO ins Array packet() sowie packetheader()
	Read_Ethernet_Frame (packet,&packetlen);	
	
	//wurde ein ARP packet empfangen?
	//2 Byte fr Nutzlastbereich (0806 fr ARP)
	if(packet[ENETPACKETTYPE0] == 0x08 && packet[ENETPACKETTYPE1] == 0x06)
		{
		if (IfArp(packet) == 1)
			{
			//printf("ARP()\n");
			//Erzeugt ein ARP Reply Packet
			ARP_Reply(packet,&packetlen);
			}
		}
	
	//wurde ein ICMP, UDP oder TCP packet empfangen?
	//2 Byte fr Nutzlastbereich (0800 fr IP) 
	else if(packet[ENETPACKETTYPE0] == 0x08 && packet[ENETPACKETTYPE1] == 0x00

		//Ist das IP Packet fr meine IP Addresse bestimmt
		//Vergleiche IP Destaddr mit meiner IP
		&& packet[IP_DESTADDR] == MYIP[0]
		&& packet[IP_DESTADDR+1] == MYIP[1]
		&& packet[IP_DESTADDR+2] == MYIP[2]
		&& packet[IP_DESTADDR+3] == MYIP[3])
		{
		//Ist protokoll Byte = 1 dann ist es ein ICMP Packet
		if(packet[IP_PROTO] == PROT_ICMP)
			{
			//printf("ICMP()\n");
			//Erzeugt ein ICMP Reply Packet (PING Echo)
			ICMP_Reply(packet,packetlen);
			}
		
		//Ist protokoll Byte = 17 dann ist es ein UDP Packet
		else if(packet[IP_PROTO] == PROT_UDP)
			{
			printf("udp() \n");
			}
		//Ist protokoll Byte = 6 dann ist es ein TCP Packet
		else if(packet[IP_PROTO] == PROT_TCP)
			{

			//printf("tcp() \n");

			if (TCP_OPEN(packet,&packetlen)==1)
				{
				return(0);
				}
			
			if (packet[TCP_DESTPORT] == 0 && packet[TCP_DESTPORT+1] == 80)
				{
				httpd (packet,&packetlen);
				}
				
			if (packet[TCP_DESTPORT] == 0 && packet[TCP_DESTPORT+1] == 23)
				{
				telnet (packet,&packetlen);
				}
			}
	}
return (0);
}


